@@ -1,8 +1,7 @@
 """
 One of the primary purposes of mokapot is to assign confidence estimates to PSMs.
 This task is accomplished by ranking PSMs according to a score or metric and
-using an appropriate confidence estimation procedure for the type of data
-(currently, linear and cross-linked PSMs are supported). In either case,
+using an appropriate confidence estimation procedure for the type of data.
 mokapot can provide confidence estimates based any score, regardless of
 whether it was the result of a learned :py:func:`mokapot.model.Model`
 instance or provided independently.
@@ -11,10 +10,10 @@ The following classes store the confidence estimates for a dataset based on the
 provided score. In either case, they provide utilities to access, save, and
 plot these estimates for the various relevant levels (i.e. PSMs, peptides, and
 proteins). The :py:func:`LinearConfidence` class is appropriate for most
-proteomics datasets, whereas the :py:func:`CrossLinkedConfidence` is
-specifically designed for crosslinked peptides.
+proteomics datasets.
 """
 import os
+import copy
 import logging
 
 import pandas as pd
@@ -29,6 +28,111 @@ LOGGER = logging.getLogger(__name__)
 
 
 # Classes ---------------------------------------------------------------------
+class GroupedConfidence:
+    """
+    Performed grouped confidence estimation for a collection of PSMs.
+
+    Parameters
+    ----------
+    psms : LinearPsmDataset object
+        A collection of PSMs.
+    scores : np.ndarray
+        A vector containing the score of each PSM.
+    desc : bool
+        Are higher scores better?
+    eval_fdr : float
+        The FDR threshold at which to report performance. This parameter
+        has no affect on the analysis itself, only logging messages.
+    """
+
+    def __init__(self, psms, scores, desc=True, eval_fdr=0.01):
+        """Initialize a GroupedConfidence object"""
+        group_psms = copy.copy(psms)
+        self.group_column = group_psms._group_column
+        group_psms._group_column = None
+        scores = scores * (desc * 2 - 1)
+
+        # Do TDC
+        scores = (
+            pd.Series(scores, index=psms._data.index)
+            .sample(frac=1)
+            .sort_values()
+        )
+
+        idx = (
+            psms.data.loc[scores.index, :]
+            .drop_duplicates(psms._spectrum_columns, keep="last")
+            .index
+        )
+
+        self.group_confidence_estimates = {}
+        for group, group_df in psms._data.groupby(psms._group_column):
+            LOGGER.info("Group: %s == %s", self.group_column, group)
+            group_psms._data = None
+            tdc_winners = group_df.index.intersection(idx)
+            group_psms._data = group_df.loc[tdc_winners, :]
+            group_scores = scores.loc[group_psms._data.index].values + 1
+            res = group_psms.assign_confidence(
+                group_scores * (2 * desc - 1), desc=desc, eval_fdr=eval_fdr
+            )
+            self.group_confidence_estimates[group] = res
+
+    def to_txt(self, dest_dir=None, file_root=None, sep="\t", decoys=False):
+        """
+        Save confidence estimates to delimited text files.
+
+        Parameters
+        ----------
+        dest_dir : str or None, optional
+            The directory in which to save the files. `None` will use the
+            current working directory.
+        file_root : str or None, optional
+            An optional prefix for the confidence estimate files. The
+            suffix will always be `mokapot.psms.txt` and
+            `mokapot.peptides.txt`.
+        sep : str, optional
+            The delimiter to use.
+        decoys : bool, optional
+            Save decoys confidence estimates as well?
+
+        Returns
+        -------
+        list of str
+            The paths to the saved files.
+        """
+        ret_files = []
+        for group, res in self.group_confidence_estimates.items():
+            prefix = file_root + f".{group}"
+            new_files = res.to_txt(
+                dest_dir=dest_dir, file_root=prefix, sep=sep, decoys=decoys
+            )
+            ret_files.append(new_files)
+
+        return ret_files
+
+    def __repr__(self):
+        """Nice printing."""
+        ngroups = len(self.group_confidence_estimates)
+        lines = [
+            "A mokapot.confidence.GroupedConfidence object with "
+            f"{ngroups} groups:\n"
+        ]
+
+        for group, conf in self.group_confidence_estimates.items():
+            lines += [f"Group: {self.group_column} == {group}"]
+            lines += ["-" * len(lines[-1])]
+            lines += [str(conf)]
+
+        return "\n".join(lines)
+
+    def __getattr__(self, attr):
+        """Make groups accessible easily"""
+        try:
+            return self.grouped_confidence_estimates[attr]
+        except KeyError:
+            raise AttributeError
+
+
 class Confidence:
     """
     Estimate and store the statistical confidence for a collection of
@@ -90,9 +194,9 @@ class Confidence:
             An optional prefix for the confidence estimate files. The
             suffix will always be `mokapot.psms.txt` and
             `mokapot.peptides.txt`.
-        sep : str
+        sep : str, optional
             The delimiter to use.
-        decoys : bool
+        decoys : bool, optional
             Save decoys confidence estimates as well?
 
         Returns
@@ -212,7 +316,6 @@ class LinearConfidence(Confidence):
 
     def __init__(self, psms, scores, desc=True, eval_fdr=0.01):
         """Initialize a a LinearPsmConfidence object"""
-        LOGGER.info("=== Assigning Confidence ===")
         super().__init__(psms, scores, desc)
         self._target_column = _new_column("target", self._data)
         self._data[self._target_column] = psms.targets
@@ -283,7 +386,7 @@ class LinearConfidence(Confidence):
             proteins = picked_protein(
                 peptides,
                 self._target_column,
-                self._peptide_column[0],
+                self._peptide_column,
                 self._score_column,
                 self._proteins,
             )
@@ -297,6 +400,11 @@ class LinearConfidence(Confidence):
             ).reset_index(drop=True)
             scores = data.loc[:, self._score_column].values
             targets = data.loc[:, self._target_column].astype(bool).values
+            if all(targets):
+                LOGGER.warning(
+                    "No decoy PSMs remain for confidence estimation. "
+                    "Confidence estimates may be unreliable."
+                )
 
             # Estimate q-values and assign to dataframe
             LOGGER.info("Assiging q-values to %s...", level)
@@ -320,9 +428,16 @@ class LinearConfidence(Confidence):
 
             # Calculate PEPs
             LOGGER.info("Assiging PEPs to %s...", level)
-            _, pep = qvality.getQvaluesFromScores(
-                scores[targets], scores[~targets], includeDecoys=True
-            )
+            try:
+                _, pep = qvality.getQvaluesFromScores(
+                    scores[targets], scores[~targets], includeDecoys=True
+                )
+            except SystemExit as msg:
+                print(msg)
+                if "no decoy hits available for PEP calculation" in str(msg):
+                    pep = 0
+                else:
+                    raise
 
             level = level.lower()
             data["mokapot PEP"] = pep
